
// ScanController Move sketch  - for Adafruit Motorshield
// provided input should be 10 or 14 bytes 
// byte 0    = 'F' or 'M' Fullsteps or Microsteps 
// byte 1    = 'F' or 'B' Forewards or Backwards 
// byte 2345 = number of steps 0001 to 9999
// byte 6789 = speed
// OPTIONAL: only usefull with noisy connection : byte 10,11,12,13 = CRC16 checksum
#include <AFMotor.h>


// Connect a stepper motor with 48 steps per revolution
// to motor port #2 (M3 and M4)
AF_Stepper motor(96, 2);
// Connect a DC motor with  to motor port M1
AF_DCMotor motorDC(1);

void setup() {
  // initialize serial:
  Serial.begin(9600);

  // turn on motor
  motorDC.setSpeed(200);
  motorDC.run(RELEASE);
}

void loop() {
  static char buffer[80];
  uint16_t crc_out;

  int endpos = readline(Serial.read(), buffer, 80) ; 
  if ( endpos > 0) {
    //if (crcverify (buffer, endpos)){ // Uncomment for CRC check
    // Valid Command with good CRC received or no CRC checked 
    Serial.println("OK OK" );
    String command(buffer);
    int nrsteps = command.substring(2,6).toInt();
    motor.setSpeed(command.substring(6,10).toInt());
    if  (command.substring(1,2) != "F"){
      motorDC.run(FORWARD);
      motorDC.setSpeed(255);  
    }

    motor.step(nrsteps,
    (command.substring(1,2) == "F" ? FORWARD : BACKWARD ),
    (command.substring(0,1) == "F" ? DOUBLE : INTERLEAVE )
      );

    if  (command.substring(1,2) != "F"){
      motorDC.run(RELEASE);
    }
    Serial.println("MoveEnd MoveEnd");

    // }// Uncomment for CRC check
    // else{// Uncomment for CRC check
    //   Serial.println("ER ER"); // Uncomment for CRC check
    // }// Uncomment for CRC check
  }
}

// Helper Functions Below 

// CRC16 Function
int crcverify (char* message, unsigned long length)
{
  if (length < 5)
    return false;
  /*
   * Returns true if the last four chars in a message converted from hex to int 
   * are the crc of the preceding bytes.
   */

  unsigned short expected = crc16_ccitt(message, length - 4);
  String str(message);
  int number = (int)strtol( &str[length -4], NULL, 16);
  if (expected != number)
    Serial.println(String("FAILED CHECKSUM: Expected ") + expected + String(" received ") +  number);
  return expected == number;
}

unsigned short crc16_ccitt(char *buf, int size)
{
  unsigned short crc = 0;
  while (--size >= 0) {
    int i;
    crc ^= (unsigned short) *buf++ << 8;
    for (i = 0; i < 8; i++)
      if (crc & 0x8000)
        crc = crc << 1 ^ 0x1021;
      else
        crc <<= 1;
  }
  return crc;
}

// Readline Helper Function
int readline(int readch, char *buffer, int len)
{
  static int pos = 0;
  int rpos;

  if (readch > 0) {
    switch (readch) {
    case '\n': // Ignore new-lines
    case '\r': // Return on CR
      rpos = pos;
      pos = 0;  // Reset position index ready for next time
      return rpos;
    default:
      if (pos < len-1) {
        buffer[pos++] = readch;
        buffer[pos] = 0;
      }
    }
  }
  // No end of line has been found, so return -1.
  return -1;
}

